#!/bin/bash

ffoo import core
ffoo import flow
ffoo import recon
ffoo import testing
ffoo import yummy


OLCS_RUNHOW=$(iniread -s mk.run.how mk-ol.ini)
OLCS_VGLOG="cimserver-valgrind.log"
OLCS_VG="valgrind --trace-children=yes --log-file=$OLCS_VGLOG"
OLCS_GDBFIFO="/tmp/gdbfifo.$$"

disable_pam_for() {
    echo "disabling PAM for $1"
    sed -i -e 's/^[^#].* pam_/# \0/' /etc/pam.d/$1
}

start_cimserver() {
    #
    # Start CIMOM in a manner defined by OLCS_RUNHOW
    #
    local cmd="cimserver"
    local args="daemon=false"
    local out="cimserver.out"
    local err="cimserver.err"
    register_artifact "$out" "$err"
    case $OLCS_RUNHOW in
        simple)
            $cmd $args > $out 2>$err &
            ;;
        service)
            systemctl start tog-pegasus
            ;;
        gdb)
            yum_install_if_needed gdb
            rm -f $OLCS_GDBFIFO
            mkfifo $OLCS_GDBFIFO
            gdb cimserver 2>$err < $OLCS_GDBFIFO | tee $out &
            echo  "run $args" > $OLCS_GDBFIFO
            ;;
        valgrind)
            yum_install_if_needed valgrind
            disable_pam_for wbem
            chmod -s /usr/sbin/unix_chkpwd
            register_artifact "$OLCS_VGLOG"
            $OLCS_VG $cmd $args > $out 2>$err &
            ;;
    esac
    wait_until -t 20 listening_on 5989
}

stop_cimserver() {
    #
    # Stop CIMOM according to how it was run (OLCS_RUNHOW)
    #
    echo "stopping cimserver"
    case $OLCS_RUNHOW in
        simple)
            killall cimserver
            ;;
        service)
            systemctl stop tog-pegasus
            ;;
        gdb)
            killall cimserver
            echo bt > $OLCS_GDBFIFO
            echo quit > $OLCS_GDBFIFO
            echo y > $OLCS_GDBFIFO
            rm -f $OLCS_GDBFIFO
            ;;
        valgrind)
            killall -r ^memcheck-
            ;;
    esac
    wait_until -t 20 -e ! pids_matching cimserver,^memcheck-
}

comment_cimserver() {
    #
    # Present a human-readable version of OLCS_RUNHOW
    #
    case $OLCS_RUNHOW in
        simple)
            echo "as a separate process"
            ;;
        service)
            echo "as a service"
            ;;
        gdb|valgrind)
            echo "under $OLCS_RUNHOW"
            ;;
    esac
}

